import java.awt.*;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Point2D.Float;
import java.awt.image.BufferedImage;
import java.io.IOException;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;

import com.mindfusion.diagramming.*;

public class MainFrame extends JFrame
{
	private Diagram diagram;
	private DiagramView diagramView;
	
	private static final long serialVersionUID = 1L;

	public MainFrame()
	{
		initDiagram();
		initFrame();
	}

	private void initDiagram()
	{
		diagram = new Diagram();
		diagram.setLinkHeadShape(ArrowHeads.Arrow);
		diagram.setLinkHeadShapeSize(10);
		diagram.setAdjustmentHandlesSize(4);
		diagram.setBackBrush(new SolidBrush(Color.white));
		diagram.setBackgroundImageAlign(ImageAlign.TopLeft);
		diagram.setDynamicLinks(true);
		diagram.setMeasureUnit(GraphicsUnit.Point);
		diagram.setLinkShape(LinkShape.Bezier);

		diagram.addDiagramListener(new DiagramAdapter()
		{
			public void clicked(DiagramEvent e)
			{	
				onClicked(e);
			}
			
			@Override
			public void linkDoubleClicked(LinkEvent linkEvent)
			{
				onLinkDoubleClicked( linkEvent );
			}
		});
		
		diagramView = new DiagramView(diagram);
		diagramView.setBehavior(Behavior.LinkShapes);
		diagramView.setDiagram(diagram);
		diagramView.setInplaceEditFont(new Font("Microsoft Sans Serif", Font.PLAIN, 8));
		diagramView.setModificationStart(ModificationStart.AutoHandles);
		diagramView.getPrintOptions().setRenderTarget(RenderTarget.Printer);

		// Enable serialization of IconNode instances
		Diagram.registerItemClass(IconNode.class.getClass(), "IconNode", 1);
		
		// Let users draw IconNode objects
		diagramView.setCustomNodeType(IconNode.class);
		diagramView.setBehavior(Behavior.Custom);
		
		// Show some info about this sample
		ShapeNode info = new ShapeNode(diagram);
		info.setBounds(50, 50, 400, 120);
		info.setText("This example shows how to create a custom node class that " +
			"exposes some new properties in addition to the ones defined by " +
			"DiagramNode and implements its own drawing logic. Click on the " +
			"drawing surface or draw with the mouse to create instances of " +
			"the custom IconeNode class.");
		info.setBrush(new SolidBrush(Color.white));
		info.setLocked(true);
		info.setFont(new Font("Arial", Font.PLAIN, 10));
		info.resizeToFitText(FitSize.KeepRatio);
		info.setTransparent(true);
		diagram.getNodes().add(info);
		

	}

	public void onLinkDoubleClicked( LinkEvent event )
	{
		DiagramLink link = event.getLink();
		// Edit the label
		//link.setText( JOptionPane.showInputDialog( this, "Edit the label", link.getText()) ); // one label
		//LinkLabel theLabel = link.addLabel( JOptionPane.showInputDialog( this, "Edit the label", link.getText()) ); // many labels but appear in exactly the same place!
		PointList pointsOnLink = link.getControlPoints();
		Float[] points = pointsOnLink.getArray();
		Rectangle2D rectangle2D = link.getEditRect( points[points.length / 2] );
		ShapeNode label;


		if (!(link.getSubordinateGroup() == null) && !(link.getSubordinateGroup().getAttachedNodes() == null))
		{
			// assume only one linked 'label' node
			DiagramNodeList diagramNodesList = link.getSubordinateGroup().getAttachedNodes();
			label = (ShapeNode)diagramNodesList.get( 0 );
		}
		else
		{
			label = diagram.getFactory().createShapeNode( rectangle2D );
			label.attachTo( link, AttachToLink.Segment,points.length -1 );
			label.setTransparent( true );
			label.setIgnoreLayout( true );
			label.setAllowIncomingLinks( false );
			label.setAllowOutgoingLinks( false );
		}
		label.setText(JOptionPane.showInputDialog( this, "Edit the label", label.getTextToEdit()));

	}
	


	private Image drawSign()
	{
		BufferedImage background = new BufferedImage(1000, 1000, BufferedImage.TYPE_INT_ARGB);
		Graphics2D graphics = background.createGraphics();

		graphics.dispose();

		return background;
	}
	
	public void onClicked(DiagramEvent e)
	{
		IconNode iconNode = new IconNode(diagram);

		iconNode.moveTo(e.getPointerPosition().x, e.getPointerPosition().y);

		diagram.getNodes().add(iconNode);
		diagram.setBackgroundImage( drawSign() );
		repaint();
	}
	
	public static Rectangle2D fromLTRB(double l, double t, double r, double b)
	{
		return new Rectangle2D.Double(l, t, r - l, b - t);
	}
	
	private void initFrame()
	{
		Dimension size = Toolkit.getDefaultToolkit().getScreenSize();
		setSize(size);
		setTitle("MindFusion.Diagramming Sample: Icon Nodes");
		
		addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent e)
			{
				System.exit(0);
			}
		});

		JScrollPane scrPane = new JScrollPane(diagramView); 
		setLayout(new GridLayout(1,1));
		add(scrPane);
	}
	
	
}